/* This file is part of the model extraction module to FaCT++
Copyright (C) 2003-2008 by Johannes Bauer

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#ifndef _JNIMODELBUILDER_H
#define _JNIMODELBUILDER_H

#include <iostream>
#include <sstream>
#include <vector>
#include <set>

#include "Kernel.h"

#include <jni.h>

#include "globaldef.h"
#include "dlCompletionTree.h"
#include "dlCompletionTreeArc.h"
#include "CGLabel.h"
#include "logging.h"
#include "dlDag.h"
#include "tRole.h"

#include "JNIActor.h"
#include "JNISupport.h"


class ModelBuilderCPP {
  private:
  map<const DlCompletionTree*, pair<jstring,short>*> seenNodes;
	map<const TConcept*, set<const TConcept*>*> synMap;

  int indivCount;

  JNIEnv *env;
	DLDag *DLHeap;
  jobject builder;
  jobject instance;
  jobject translator;
  jclass modelBuilderJ;
  jclass translatorJ;
  jmethodID pushObjectPropertyID;
  jmethodID popObjectPropertyID;
  jmethodID addConceptID;
  jmethodID addNegatedConceptID;
  jmethodID pruneID;
  jmethodID initModelID;
  jmethodID aboxInconsistentID;
  jmethodID conceptMandatoryID;
  jmethodID roleMandatoryID;
  jmethodID translateID;
  jmethodID setBlockerID;

  const char* exc;

  public:
  ModelBuilderCPP(JNIEnv * env, jobject builder, jobject instance);
  ~ModelBuilderCPP();

  private:

  /// A recursive method calling methods of the Java class ModelBuilder to build a model.
  void buildModel(const DlCompletionTree* tree, int depth);

	void setupSynonyms(const ReasoningKernel*);

  jstring getNameFor(const DlCompletionTree* tree);
	short getStatus(const DlCompletionTree* tree);
	void setStatus(const DlCompletionTree* tree, short status);
	static const short UNSEEN = 0;
	static const short EXPLORED = 1;
	static const short PRUNED = 2;
	static const short EXPLORING = 2;

  ///////////////////
  // java method wrappers
  ///////////////////

  public:
  void initModel(const DlCompletionTree* tree, const ReasoningKernel* kernel, DLDag* DLHeap, int depth);
  const char* getException();
  void aboxInconsistent();

  private:
  void pushObjectProperty(const char* name, bool mandatory, jstring object);
  void popObjectProperty();
  void addConcept(const char* name);
  void addConcept(BipolarPointer bp, bool mandatory, map<set<const TConcept*>*, short> &label);
  void addNegatedConcept(const char* name);
  void prune(const DlCompletionTree* tree);
  void conceptMandatory();
  void roleMandatory();
  void noteException(const char* exc);
	void setBlocker(const DlCompletionTree* blocker);
};

#endif
