/* This file is part of the FaCT++ DL reasoner
Copyright (C) 2003-2008 by Dmitry Tsarkov

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include "CompletionTree.h"


CompletionTree::CompletionTree(const DlCompletionTree* tree, int depth)
{
  CompletionTree(tree, depth, NULL);
}

CompletionTree::CompletionTree(const DlCompletionTree* tree, int depth, CompletionTree* parent)
{
  if(parent != NULL) {
    seenNodes = parent->seenNodes;
  } else {
    seenNodes = new(std::map<const DlCompletionTree*,CompletionTree*>);
  }

  (*seenNodes)[tree] = this;

  DlCompletionTree::const_edge_iterator p, p_end;
  const TRole* role;
  std::set<const CompletionTree*>* objects;
  if(depth != 0)
  for ( p = tree->begins(), p_end = tree->ends(); p < p_end; ++p ) {
    role = (*p)->getRole();
    objects = getObjects( role );
    if( objects == NULL ) {
      objects = new(std::set<const CompletionTree*>);
      arcMap[role] = objects;
    }

    CompletionTree* object = seenNodes->find((*p)->getArcEnd())->second;
    if(object == NULL)
      objects->insert(new CompletionTree((*p)->getArcEnd(), depth - 1, this));
    else
      objects->insert(object);
  }
}

CompletionTree::~CompletionTree()
{
  seenNodes->erase(theTree);
  if( seenNodes->size() == 0 )
    delete(seenNodes);

  std::map<const TRole*, std::set<const CompletionTree*>* >::const_iterator alpha, omega;
  for( alpha = arcMap.begin(), omega = arcMap.end(); alpha != omega ; ++alpha)
    delete (alpha->second);
}

