/* This file is part of the FaCT++ DL reasoner
Copyright (C) 2003-2008 by Dmitry Tsarkov

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#ifndef _JNICOMPLETIONTREE_H
#define _JNICOMPLETIONTREE_H

#include <vector>
#include <map>
#include <set>
#include <iostream>

#include "globaldef.h"
#include "dlCompletionTreeArc.h"
#include "tSaveList.h"
#include "tRestorer.h"
#include "CGLabel.h"
#include "dlCompletionTree.h"
#include "logging.h"

/// A JNI-enabled representation of a completion tree computed for some concept.
class CompletionTree { 

  public:
   typedef std::map<const TRole*, std::set<const CompletionTree*>*> ArcMap;
   static long newRoot(const DlCompletionTree* tree, int depth)
   {
     if(currentRoot != NULL)
        delete currentRoot;
     currentRoot = new CompletionTree(tree, depth);
     return (long) currentRoot->theTree;
   }

  private:
   static CompletionTree* currentRoot;

  private:
    std::map<const DlCompletionTree*,CompletionTree*>* seenNodes;
    DlCompletionTree* theTree;
    ArcMap arcMap;

    CompletionTree(const DlCompletionTree* tree, int depth);
    CompletionTree(const DlCompletionTree* tree, int depth, CompletionTree* parent);
    ~CompletionTree();

    long treeId;

//    const std::vector<TRole*>* getRoles() { return roles; };
    std::set<const CompletionTree*>* getObjects(const TRole* role) { return (arcMap.find(role))->second; }

};

#endif
